from math import pi
from cmath import phase
import numpy as np
from scipy.signal import freqz, lfilter
import matplotlib.pyplot as plt

# Mark Tillotson, 2017-07-19
# Digital filter plotting functions

def plot_response (b, a, fs, logarithmic = True):
    f, r = freqz (b, a)
    m, ph = abs(r), np.vectorize (phase)(r)
    if logarithmic:
        plt.semilogx (f / (2*pi) * fs, 20 * np.log10(m))
    else:
        plt.plot (f / (2*pi) * fs, 20 * np.log10(m))
    plt.xlabel('Frequency')
    plt.ylabel('Amplitude response [dB]')
    plt.ylim (-80, 10)
    plt.grid()
    plt.show()


def plot_everything (b, a, fs):
    w, h = freqz (b, a, whole = True)
    h_dB = 20 * np.log10 (abs(h))
    plt.subplot (4, 1, 1)
    plt.plot (w*fs/2/pi, h_dB)
    plt.xlim (0.0, fs)
    plt.ylim (-90, 5)
    plt.ylabel ('Magnitude (db)')
    plt.xlabel (r'Frequency')
    plt.title (r'Frequency response')
    plt.subplot (4, 1, 2)
    h_Phase = np.unwrap (np.arctan2 (np.imag(h), np.real(h)))
    plt.plot (w * fs/pi/2, h_Phase)
    plt.xlim (0.0, fs)
    plt.ylabel ('Phase (radians)')
    plt.xlabel (r'Frequency')
    plt.title (r'Phase response')
    plt.subplots_adjust (hspace = 0.5)
    steps = 500
    impulse = np.repeat (0.0, steps)
    impulse[0] =1.0
    x = np.arange (0, steps)
    response = lfilter (b, a, impulse)
    plt.subplot (4, 1, 3)
    plt.step (x, response)
    plt.ylabel ('Amplitude')
    plt.xlabel (r'n (samples)')
    plt.axhline (0)
    plt.title (r'Impulse response')
    plt.subplot (4, 1, 4)
    step = np.cumsum (response)
    plt.step (x, step)
    plt.ylabel ('Amplitude')
    plt.xlabel (r'n (samples)')
    plt.title (r'Step response')
    plt.axhline (0)
    plt.subplots_adjust (hspace = 0.5)
    plt.show()

"""
////////////////////////////////////////////////////////////////////////////////////////////
//                                TERMS OF USE: MIT License
////////////////////////////////////////////////////////////////////////////////////////////
// Permission is hereby granted, free of charge, to any person obtaining a copy of this 
// software and associated documentation files (the "Software"), to deal in the Software 
// without restriction, including without limitation the rights to use, copy, modify, merge,
// publish, distribute, sublicense, and/or sell copies of the Software, and to permit 
// persons to whom the Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or
// substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
// PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
// FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR 
// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
// DEALINGS IN THE SOFTWARE.
////////////////////////////////////////////////////////////////////////////////////////////
"""
