from scipy.signal import ZerosPolesGain

# Mark Tillotson, 2017-07-19
# Digital filter utilities

def quantize (value, bits):
    if isinstance (value, complex):
        return quantize (value.real, bits) + 1j * quantize (value.imag, bits)
    assert isinstance (value, float)
    if bits is None:
        return value.real
    ratio = 1.0 * (1 << bits)
    int_value = int (ratio * value.real)
    return int_value / ratio



def combine_responses (f1, f2):
    f1 = f1.to_zpk ()
    f2 = f2.to_zpk ()
    z1 = [ zero for zero in f1.zeros ]
    z2 = [ zero for zero in f2.zeros ]
    p1 = [ pole for pole in f1.poles ]
    p2 = [ pole for pole in f2.poles ]
    return ZerosPolesGain (z1+z2, p1+p2, f1.gain*f2.gain)


def factorize_poly (roots):
    result_conjugate_pairs = []
    result_reals = []
    result_compl = []
    for root in roots:
        if root.imag == 0.0:
            result_reals.append (root)
        if root.imag != 0.0:
            matched = False
            for other in roots:
                if other.real == root.real and other.imag + root.imag == 0:
                    matched = True
                    if root.imag > 0:
                        result_conjugate_pairs.append (root)
            if not matched:
                result_compl.append (root)

    return result_conjugate_pairs, result_reals, result_compl

"""
////////////////////////////////////////////////////////////////////////////////////////////
//                                TERMS OF USE: MIT License
////////////////////////////////////////////////////////////////////////////////////////////
// Permission is hereby granted, free of charge, to any person obtaining a copy of this 
// software and associated documentation files (the "Software"), to deal in the Software 
// without restriction, including without limitation the rights to use, copy, modify, merge,
// publish, distribute, sublicense, and/or sell copies of the Software, and to permit 
// persons to whom the Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or
// substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
// PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
// FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR 
// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
// DEALINGS IN THE SOFTWARE.
////////////////////////////////////////////////////////////////////////////////////////////
"""
